﻿#include "renri_tm1638.h"

const byte NUM[] = {
      0b00111111,     // 0
      0b00000110,     // 1
      0b01011011,     // 2
      0b01001111,     // 3      
      0b01100110,     // 4
      0b01101101,     // 5      
      0b01111101,     // 6
      0b00100111,     // 7  
      0b01111111,     // 8
      0b01101111,     // 9
      0b00111111,     // a
      0b00000110,     // b 
      0b00111111,     // c
      0b00000110,     // d
      0b00111111,     // e
      0b00000110     // f
};


RENRI_TM1638::RENRI_TM1638(byte sdio,byte sclk,byte sstb,byte bri){
  dio = sdio;
  clk = sclk;
  stb = sstb;
  for (int i=0; i<8; i++){
    seg7[i] = 0;
    led[i] = 0;
  }
  for (int i=0; i<4; i++){
    inData[i] = 0;
  }
  bright = bri;
  pinMode(stb,OUTPUT);
  pinMode(clk,OUTPUT);
  pinMode(dio,OUTPUT);  
}

RENRI_TM1638::RENRI_TM1638(byte sdio,byte sclk,byte sstb){
  dio = sdio;
  clk = sclk;
  stb = sstb;
  for (int i=0; i<8; i++){
    seg7[i] = 0;
    led[i] = 0;
    if(i<4){
        inData[i] = 0;
    }
  }

  pinMode(stb,OUTPUT);
  pinMode(clk,OUTPUT);
  pinMode(dio,OUTPUT);  
}

void RENRI_TM1638::SendCmd(byte data){
    WriteData(data); // DisplayCommandSet + DiplaySwitch + bright
}

void RENRI_TM1638::SendCmdSet(byte data){
    Enable();
    WriteData(data | bright); // DisplayCommandSet + DiplaySwitch + bright
    Disable();   
}

void RENRI_TM1638::SetBri(byte brightness){
  if (brightness >= 0x07){
    bright = 0x07;
  } else if(brightness <=0x00){
    bright = 0x00;  
  } else {
    bright = brightness;    
  }
}

void RENRI_TM1638::Write(char data){
  if (data){
      digitalWrite(dio,HIGH);
  } else {
      digitalWrite(dio,LOW);
  }
  delayMicroseconds(WAITTIME);
  digitalWrite(clk,LOW);
  delayMicroseconds(WAITTIME);
  digitalWrite(clk,HIGH);
  delayMicroseconds(WAITTIME);
}

void RENRI_TM1638::WriteData(char senddata){
  int i;
  char outdata = senddata; // 
  
  pinMode(dio,OUTPUT);

  for(i=0; i<8; i++){
    Write(outdata & 0x01);
    outdata >>= 1;
  }
}

void RENRI_TM1638::SetVal(byte pos,int data,boolean dp){
  byte dpdata;
  if (dp){
    dpdata=0x80;
  } else {
    dpdata=0;    
  }
  seg7[pos] = NUM[data] | dpdata;

//  Enable();
  switch(pos){
    case 0:
    case 1:
    case 2:
    case 3:
    case 4:
    case 5:
    case 6:
    case 7:
      SetOutput( (pos<<1) ,seg7[pos]);      
      
      break;
    default:
      return;
  }
//  Disable();
}

void RENRI_TM1638::SetDeg(byte pos,char data,boolean dp){
  if (dp){
    data |= 0x80;
  }
    
  seg7[pos] = data;

//  Enable();
  switch(pos){
    case 0:
    case 1:
    case 2:
    case 3:
    case 4:
    case 5:
    case 6:
    case 7:
      SetOutput( (pos<<1) ,seg7[pos]);      
      break;
    default:
      return;
  }
//  Disable();
}

void RENRI_TM1638::SetLed2(byte pos,char data,boolean dp){
  while(flag){
    delay(0);
  }
  flag=true;

  Enable();
  delayMicroseconds(WAITTIME);
  SendCmd(0x40);      // 出力の設定
  Disable();
  delayMicroseconds(WAITTIME);
  SetLed(pos,data,dp);
  delayMicroseconds(WAITTIME);
  Disable();

  flag=false;  
}

void RENRI_TM1638::SetLed(byte pos,char data,boolean dp){
  if (dp){
    led[pos] |= data;
  } else {
    led[pos] &= ~data;
  }

  switch(pos){
    case 0:
    case 1:
    case 2:
    case 3:
    case 4:
    case 5:
    case 6:
    case 7:
      SetOutput( (pos<<1) + 1,led[pos]);      
      break;
    default:
      return;
  }
}

void RENRI_TM1638::SetOutput(byte pos,char data){
  Enable();
  WriteData(0xc0 | pos); // Address Set
  delayMicroseconds(1);
  WriteData(data);  
  delayMicroseconds(1);
  Disable();
}

void RENRI_TM1638::SetIntToDisp(byte spos,int keta,int valdata){
  int divdata=10000;
  int tmp;
  
  if ((keta <= 5) && (keta > 0)){
    while(flag){
      delay(0);
    }
    flag=true;
    SendCmdSet(0x88);    // Display Control ShowTurn On
    Enable();  
    SendCmd(0x40);      // 出力の設定
    Disable();
    for (int i=5; i>keta; i--){
      divdata /= 10;
    }
    for (int i=0; i<keta; i++){
      tmp = (valdata / divdata) % 10;
      
      SetVal(spos,tmp);
      divdata /= 10;
      spos++;
    }
    Disable();
    SendCmdSet(0x88);  // Display Control ShowTurn On
    flag=false;
  } else {
    return;
  }
}

void RENRI_TM1638::SetLngToDisp(byte spos,int keta,long valdata){
  long divdata=10000000;
  long tmp;
  if ((keta <= 8) && (keta > 0)){
    while(flag){
      delay(0);
    }
    flag=true;
    SendCmdSet(0x88);    // Display Control ShowTurn On
    Enable();
    SendCmd(0x40);      // 出力の設定
    Disable();
    for (int i=8; i>keta; i--){
      divdata /= 10;
    }
    for (int i=0; i<keta; i++){
      tmp = (valdata / divdata) % 10;
      
      SetVal(spos,tmp);
      divdata /= 10;
      spos++;
    }
    Disable();
    SendCmdSet(0x88);  // Display Control ShowTurn On
    flag=false;
  } else {
    return;
  }
}

char RENRI_TM1638::Read(){
    char data;
    char bkdata;
    char count;

    count=0;

    delayMicroseconds(WAITTIME);
    digitalWrite(clk,LOW);
    delayMicroseconds(WAITTIME);
    bkdata = digitalRead(dio);
    do{
      data = digitalRead(dio);
      if(bkdata==data){
        count++;
      } else {
        count=0;
      }
      bkdata = data;
    } while(count<10);

    digitalWrite(clk,HIGH);

    delayMicroseconds(WAITTIME);    

    if (data==LOW){
      return 0x00;
    } else{
      return 0x80;
    }
}

char RENRI_TM1638::ReadData(){
  char indata;
  indata = 0;
  pinMode(dio,INPUT);  

  for(int i=0; i<8; i++){
    indata >>= 1;
    indata |= Read();    
  }
//  pinMode(dio,OUTPUT);  
  
  return indata;
}

void RENRI_TM1638::Clear(boolean NumFlag){
  while(flag){
      delay(0);
  }
  flag=true;

  for (int i=0; i<8; i++){
    seg7[i] = 0;
    led[i] = 0;
  }

  pinMode(dio,OUTPUT);

  Enable();
  SendCmd(0x40);      // 出力の設定
  Disable();

  for(int i=0; i<8; i++){
    if (NumFlag=false){
     SetDeg(i,0); // BINARY表示
     SetLed(i,0);
    } else {
     SetVal(i,0); // BCD表示
     SetLed(i,0);      
    }
  }

  Disable();  

  flag=false;

}

void RENRI_TM1638::InUpdate(){
    char tempData;
    while(flag){
      delay(0);
    }
    flag=true;
    Enable();  // 信号有効
    WriteData(0x42); // 出力を設定(0x40）入力は(0x42) 
//    Disable();
    delayMicroseconds(WAITTIME);
    for(int i=0; i<4; i++){
      tempData = ReadData();
      if (intempData[i] == tempData){
        incount[i]++;
      } else {
        incount[i]=0;
      }
      if (incount[i]>=2){
        inData[i] = tempData;
        incount[i] = 0;
      }
      intempData[i] = tempData;
    }    
    
    Disable(); // 信号無効 

    pinMode(dio,OUTPUT);  
    flag=false;
}
void RENRI_TM1638::OutUpdate(){
/*
  bool NumFlag = true;
  while(flag){
      delay(0);
  }
  flag=false;

  pinMode(dio,OUTPUT);

  SendCmdSet(0x88);    // Display Control ShowTurn On
  SendCmd(0x40);   // 出力の設定
  
  Enable();

  WriteData(0xc0); // Address Set
  delayMicroseconds(1);

  for(int i=0; i<8; i++){
    if (NumFlag==false){
     WriteData(seg7[i]);
  delayMicroseconds(1);
//     SetDeg(i,seg7[i]); // BINARY表示
//     SetLed(i,led[i]);
//     WriteData(seg7[i]);
     WriteData(led[i]);
  delayMicroseconds(1);
    } else {
//     SetVal(i,seg7[i]); // BCD表示
//     SetLed(i,led[i]);      
     WriteData(seg7[i]);
  delayMicroseconds(1);
     WriteData(led[i]);
  delayMicroseconds(1);
    }
  }
  Disable();  
  flag=false;
*/
}

byte RENRI_TM1638::getData(byte pos){
    if ( (pos >=0) && (pos <= 3)) {
      return inData[pos];
    } else {
      return 0x00;
    }
}

